/*
* Real-time Ethernet on PRU-ICSS
*
* Copyright (C) 2016 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
*
*    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

#include <xdc/std.h>
#include <xdc/runtime/System.h>
#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/knl/Task.h>

#include "console_utils.h"
#include "hw/hw_icss.h"
#include "hw/hw_icss_pru_ctrl.h"
#include "hw/hw_icss_intc.h"
#include "hw/hw_icss_cfg.h"
#include "hw/hw_icss_mii_mdio.h"
#include "hw/hw_icss_mii_rt_cfg.h"
#include "hw/hw_icss_iep.h"

#include "rte_drv.h"
#include "rte_appl_def.h"

#include "pruicss_v1.h"

t_bus_config rte_BusConfigDefault = {500000, 1000, 125000, 375000, 10000};
extern PRUICSS_Handle pruIcss1Handle;

t_tts_descriptor_params tts_desciptor[] = {{1000, 	0, 	20}, \
											{5000, 21, 	20}, \
											{9000, 41, 	20}, \
											{-1,   -1, 	-1}, \
											};


// main port descriptors - local for now
t_portParams switchPort[3];				// fixed set of 3 ports
volatile unsigned char host_event_flag = 0;

void UTIL_memcpy(UInt8* dest, const UInt8* src, int size);

void rte_SetDeviceAddress(unsigned char device_addr)
{
	UInt32 baseaddr;
	PRUICSS_HwAttrs      const    *hwAttrs;
	hwAttrs = pruIcss1Handle->hwAttrs;
	baseaddr = hwAttrs->baseAddr;

	HWREGB(baseaddr + PRU_ICSS_SHARED_RAM + STATION_SLAVE_ADDRESS_OFFSET) =  device_addr;
}


void rte_SetBusConfiguration(t_bus_config *BusConf)
{
	UInt32 baseaddr;
	PRUICSS_HwAttrs      const    *hwAttrs;
	hwAttrs = pruIcss1Handle->hwAttrs;
	baseaddr = hwAttrs->baseAddr;

    // configure IEP timing parameters
    HWREG(baseaddr + PRU_ICSS_IEP + HW_ICSS_IEP_COUNT) = 0;					// set counter to 0
    HWREG(baseaddr + PRU_ICSS_IEP + HW_ICSS_IEP_CMP0) = BusConf->bus_cycle_time;
    HWREG(baseaddr + PRU_ICSS_IEP + HW_ICSS_IEP_CMP1) = BusConf->host_event;
    HWREG(baseaddr + PRU_ICSS_IEP + HW_ICSS_IEP_CMP_CFG) = 0 | (1<<2) | (1<<1) | (1<<0);	// cmp0_rst_cnt_en, cmp_en[0] and cmp_en[1]
    // clear pending events in IEP and INTC
    HWREG(baseaddr + PRU_ICSS_IEP + HW_ICSS_IEP_CMP_STATUS) = 0xFFFF;
    HWREG(baseaddr + PRU_ICSS_INTC + HW_ICSS_INTC_SECR0) = 1<<7;

    // set values in shared memory
	HWREG(baseaddr + PRU_ICSS_SHARED_RAM + BUS_CYCLE_TIME) 	= BusConf->bus_cycle_time;
	HWREG(baseaddr + PRU_ICSS_SHARED_RAM + RT_WINDOW_OPEN) 	= BusConf->rt_window_open;
	HWREG(baseaddr + PRU_ICSS_SHARED_RAM + NRT_WINDOW_OPEN)	= BusConf->nrt_window_open;
	HWREG(baseaddr + PRU_ICSS_SHARED_RAM + NRT_LAST_TX) 	= BusConf->nrt_last_tx;
}

void rte_ClearTTSDescriptorList(t_tts_descriptor_params *TTSDescList, unsigned int TTSEntries)
{
	UInt32 baseaddr;
	PRUICSS_HwAttrs      const    *hwAttrs;
	hwAttrs = pruIcss1Handle->hwAttrs;
	baseaddr = hwAttrs->baseAddr;

	memset((void*)(baseaddr+PRU_ICSS_SHARED_RAM+TTS_DESCRIPTOR_LIST), -1 , TTSEntries * 8);
}

void rte_WriteTTSFrame(unsigned char* FrameSrcAddr, unsigned int TTSListEntry, unsigned int FrameStartTime, unsigned int FrameLength)
{
	UInt32 baseaddr;
	PRUICSS_HwAttrs      const    *hwAttrs;
	hwAttrs = pruIcss1Handle->hwAttrs;
	baseaddr = hwAttrs->baseAddr;

	// copy frame to TTS frame buffer
	memcpy((void*)(baseaddr+PRU_ICSS_SHARED_RAM+TTS_FRAME_BUFFER+TTSListEntry*0x40), (void *)FrameSrcAddr, FrameLength);
	// set TTS descriptor list
	HWREG(baseaddr+PRU_ICSS_SHARED_RAM+TTS_DESCRIPTOR_LIST+TTSListEntry*8) =  FrameStartTime;
	HWREGH(baseaddr+PRU_ICSS_SHARED_RAM+TTS_DESCRIPTOR_LIST+8+TTSListEntry*8) =  TTSListEntry*0x40;
	HWREGH(baseaddr+PRU_ICSS_SHARED_RAM+TTS_DESCRIPTOR_LIST+12+TTSListEntry*8) =  FrameLength;
}

void rte_StartStopIepTimer(unsigned int startstop)
{
	UInt32 baseaddr;
	PRUICSS_HwAttrs      const    *hwAttrs;
	hwAttrs = pruIcss1Handle->hwAttrs;
	baseaddr = hwAttrs->baseAddr;

	if(startstop!=0)
		HWREG(baseaddr + PRU_ICSS_IEP + HW_ICSS_IEP_GLOBAL_CFG) = 0x551;		// start IEP timer
	else
		HWREG(baseaddr + PRU_ICSS_IEP + HW_ICSS_IEP_GLOBAL_CFG) = 0x550;		// stop IEP timer
	return;
}


void rte_host_event_handler(void)
{
	UInt32 baseaddr, temp32;
	PRUICSS_HwAttrs      const    *hwAttrs;
	hwAttrs = pruIcss1Handle->hwAttrs;
	baseaddr = hwAttrs->baseAddr;

	// clear event in PRU-ICSS INTC
	temp32 = HWREG(baseaddr + PRU_ICSS_INTC + HW_ICSS_INTC_HIPIR2);
	HWREG(baseaddr + PRU_ICSS_INTC + HW_ICSS_INTC_SICR) = temp32;

	host_event_flag = 1;
}


/***********************************************************************/
/* utility fucntions                                                   */
/***********************************************************************/
// basic copy routine to be replaced by optimised version
void UTIL_memcpy(UInt8* dest, const UInt8* src, int size)
{
	int i;
// safe but slow, no alignment issues
	for (i=0;i<size;i++) {
		dest[i]=src[i];
	}
}




